@@ -8,7 +8,7 @@ from time import sleep, time
 
 import httplib2
 import six
-from apiclient.discovery import build
+from apiclient.discovery import build, DISCOVERY_URI
 from apiclient.errors import HttpError
 
 from bigquery.errors import (BigQueryTimeoutException, JobExecutingException,
@@ -43,7 +43,8 @@ JOB_DESTINATION_FORMAT_NEWLINE_DELIMITED_JSON = \
 JOB_DESTINATION_FORMAT_CSV = JOB_FORMAT_CSV
 
 
-def get_client(project_id, credentials=None, service_account=None,
+def get_client(project_id, credentials=None,
+               service_url=None, service_account=None,
                private_key=None, private_key_file=None,
                json_key=None, json_key_file=None,
                readonly=True, swallow_results=True):
@@ -55,6 +56,12 @@ def get_client(project_id, credentials=None, service_account=None,
         project_id: the BigQuery project id.
         credentials: an AssertionCredentials instance to authenticate requests
                      to BigQuery.
+        service_url: a URI string template pointing to the location of
+                     Google's API discovery service. Requires two parameters
+                     {api} and {apiVersion} that when filled in produce an
+                     absolute URI to the discovery document for that service.
+                     If not set then the default googleapiclient disovery URI
+                     is used.
         service_account: the Google API service account name.
         private_key: the private key associated with the service account in
                      PKCS12 or PEM format.
@@ -77,6 +84,9 @@ def get_client(project_id, credentials=None, service_account=None,
         assert (service_account and (private_key or private_key_file)) or (json_key or json_key_file), \
             'Must provide AssertionCredentials or service account and P12 key or JSON key'
 
+    if service_url is None:
+        service_url = DISCOVERY_URI
+
     if private_key_file:
         with open(private_key_file, 'rb') as key_file:
             private_key = key_file.read()
@@ -90,6 +100,7 @@ def get_client(project_id, credentials=None, service_account=None,
         private_key = json_key['private_key']
 
     bq_service = _get_bq_service(credentials=credentials,
+                                 service_url=service_url,
                                  service_account=service_account,
                                  private_key=private_key,
                                  readonly=readonly)
@@ -97,7 +108,7 @@ def get_client(project_id, credentials=None, service_account=None,
     return BigQueryClient(bq_service, project_id, swallow_results)
 
 
-def _get_bq_service(credentials=None, service_account=None, private_key=None,
+def _get_bq_service(credentials=None, service_url=None, service_account=None, private_key=None,
                     readonly=True):
     """Construct an authorized BigQuery service object."""
 
@@ -110,7 +121,7 @@ def _get_bq_service(credentials=None, service_account=None, private_key=None,
 
     http = httplib2.Http()
     http = credentials.authorize(http)
-    service = build('bigquery', 'v2', http=http)
+    service = build('bigquery', 'v2', http=http, discoveryServiceUrl=service_url)
 
     return service
 
@@ -442,6 +453,90 @@ class BigQueryClient(object):
             else:
                 return {}
 
+    def update_table(self, dataset, table, schema):
+        """Update an existing table in the dataset.
+
+        Args:
+            dataset: the dataset to update the table in.
+            table: the name of table to update.
+            schema: table schema dict.
+
+        Returns:
+            bool indicating if the table was successfully updated or not,
+            or response from BigQuery if swallow_results is set for False.
+        """
+
+        body = {
+            'schema': {'fields': schema},
+            'tableReference': {
+                'tableId': table,
+                'projectId': self.project_id,
+                'datasetId': dataset
+            }
+        }
+
+        try:
+            result = self.bigquery.tables().update(
+                projectId=self.project_id,
+                datasetId=dataset,
+                body=body
+            ).execute()
+            if self.swallow_results:
+                return True
+            else:
+                return result
+
+        except HttpError as e:
+            logging.error(('Cannot update table {0}.{1}\n'
+                           'Http Error: {2}').format(dataset, table,
+                                                     e.content))
+            if self.swallow_results:
+                return False
+            else:
+                return {}
+
+    def patch_table(self, dataset, table, schema):
+        """Patch an existing table in the dataset.
+
+        Args:
+            dataset: the dataset to patch the table in.
+            table: the name of table to patch.
+            schema: table schema dict.
+
+        Returns:
+            bool indicating if the table was successfully patched or not,
+            or response from BigQuery if swallow_results is set for False.
+        """
+
+        body = {
+            'schema': {'fields': schema},
+            'tableReference': {
+                'tableId': table,
+                'projectId': self.project_id,
+                'datasetId': dataset
+            }
+        }
+
+        try:
+            result = self.bigquery.tables().patch(
+                projectId=self.project_id,
+                datasetId=dataset,
+                body=body
+            ).execute()
+            if self.swallow_results:
+                return True
+            else:
+                return result
+
+        except HttpError as e:
+            logging.error(('Cannot patch table {0}.{1}\n'
+                           'Http Error: {2}').format(dataset, table,
+                                                     e.content))
+            if self.swallow_results:
+                return False
+            else:
+                return {}
+
     def create_view(self, dataset, view, query):
         """Create a new view in the dataset.
 
@@ -777,6 +872,7 @@ class BigQueryClient(object):
             query,
             dataset=None,
             table=None,
+            external_udf_uris=[],
             allow_large_results=None,
             use_query_cache=None,
             priority=None,
@@ -790,6 +886,11 @@ class BigQueryClient(object):
             query: required BigQuery query string.
             dataset: optional string id of the dataset
             table: optional string id of the table
+            external_udf_uris: optional list of external UDF URIs
+                    (if given,
+                        URIs must be Google Cloud Storage
+                        and have .js extensions
+                    )
             allow_large_results: optional boolean
             use_query_cache: optional boolean
             priority: optional string
@@ -835,6 +936,14 @@ class BigQueryClient(object):
         if write_disposition:
             configuration['writeDisposition'] = write_disposition
 
+        configuration['userDefinedFunctionResources'] = []
+        for external_udf_uri in external_udf_uris:
+            configuration['userDefinedFunctionResources'].append(
+                {
+                    "resourceUri": external_udf_uri
+                }
+            )
+
         body = {
             "configuration": {
                 'query': configuration
@@ -1146,7 +1255,7 @@ class BigQueryClient(object):
 
             elif col_dict['type'] == 'BOOLEAN':
                 row_value = row_value in ('True', 'true', 'TRUE')
-            
+
             elif col_dict['type'] == 'TIMESTAMP':
                 row_value = float(row_value)
 
